""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from django.db import models


class ExpressionCategory(models.Model):
    name = models.CharField(max_length=100, unique=True)
    title = models.TextField()
    description = models.TextField(blank=True, null=True)

    def __str__(self):
        return 'ExpressionCategory(name=' + str(self.name) \
            + ' ,title=' + self.title + ' ,description=' + self.description + ')'


class Expression(models.Model):
    """
    Model for jinja expressions schema
    """
    name = models.CharField(max_length=500)
    jexp = models.TextField()
    description = models.TextField(blank=True, null=True)
    tooltip = models.TextField(blank=True, null=True)
    params = models.JSONField(blank=True, default=dict, null=True)
    connector_name = models.CharField(max_length=100, blank=True, null=True)
    version = models.CharField(max_length=50, blank=True, null=True)
    action_name = models.CharField(max_length=500, blank=True, null=True)
    is_system = models.BooleanField(default=False)
    expression_category = models.ForeignKey(
        ExpressionCategory, on_delete=models.CASCADE, null=True, blank=True, related_name='operations')

    def __str__(self):
        return str(self.name)

    class Meta:
        ordering = ['name', 'version']
        unique_together = ('name', 'connector_name', 'action_name', 'version')
