""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from rest_framework import serializers

from expression_builder.models import Expression, ExpressionCategory


class ExpressionCategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = ExpressionCategory
        fields = '__all__'


class ExpressionMetaSerializer(serializers.ModelSerializer):
    expression_category = ExpressionCategorySerializer(read_only=True)

    class Meta:
        model = Expression
        fields = ('name', 'description', 'expression_category')

    def to_representation(self, instance):
        ret = super().to_representation(instance)
        return {
            'name': ret['expression_category']['name'],
            'title': ret['expression_category']['title'],
            'description': ret['expression_category']['description'],
            'operations': [
                {
                    'name': ret['name'],
                    'description': ret['description']
                }
            ]
        }


class ExpressionSerializer(serializers.ModelSerializer):
    expression_category = ExpressionCategorySerializer(read_only=True)

    class Meta:
        model = Expression
        exclude = ('id', )

    def to_representation(self, instance):
        ret = super().to_representation(instance)
        return {
            'name': ret['expression_category']['name'],
            'title': ret['expression_category']['title'],
            'description': ret['expression_category']['description'],
            'operations': [
                {
                    'name': ret['name'],
                    'jexp': ret['jexp'],
                    'description': ret['description'],
                    'tooltip': ret['tooltip'],
                    'connector_name': ret['connector_name'],
                    'action_name': ret['action_name'],
                    'version': ret['version'],
                    'params': ret['params']
                }
            ]
        }


class ExpressionIdSerializer(serializers.ModelSerializer):
    class Meta:
        model = Expression
        fields = ('id', 'name')

    def to_representation(self, instance):
        ret = super().to_representation(instance)
        ret['status'] = 'Success'
        return ret
