""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import os
from glob import glob

from rest_framework import status

from expression_builder.models import ExpressionCategory, Expression
from workflow.errors.error_constants import cs_wf_55, cs_wf_56
from workflow.exceptions import InvalidExpressionInput


def insert_jinja_record(data, result_status=[status.HTTP_200_OK], is_system=False, is_bulk=False):
    try:
        name = data.get('name')
        if not name:
            if is_bulk:
                result_status[0] = status.HTTP_207_MULTI_STATUS
                return [], [], False
            else:
                raise InvalidExpressionInput(cs_wf_55.format('expression name'))
        exp_category_instance, created = ExpressionCategory.objects.update_or_create(
            name=name, defaults={'title': data.get('title'), 'description': data.get('description')})
        exp_instances = []
        for exp in data['operations']:
            name = exp.get('name')
            if not name:
                if is_bulk:
                    result_status[0] = status.HTTP_207_MULTI_STATUS
                    continue
                else:
                    raise InvalidExpressionInput(cs_wf_55.format('operation name'))
            connector_name = exp.get('connector_name')
            action_name = exp.get('action_name')
            version = exp.get('version')
            if connector_name and action_name:
                conn_names = list(Expression.objects
                                  .filter(action_name=action_name).values_list('connector_name', flat=True))
                if conn_names and connector_name not in conn_names:
                    if is_bulk:
                        result_status[0] = status.HTTP_207_MULTI_STATUS
                        continue
                    else:
                        raise InvalidExpressionInput(cs_wf_56)
            exp.update(expression_category=exp_category_instance, is_system=is_system)
            exp_instance, created = Expression.objects.update_or_create(
                name=name,
                connector_name=connector_name,
                action_name=action_name,
                version=version,
                defaults=exp)
            exp_instances.append(exp_instance)
        return exp_category_instance, exp_instances, created
    except Exception as e:
        if is_bulk:
            result_status[0] = status.HTTP_207_MULTI_STATUS
            return [], [], False
        else:
            raise e


def insert_default_jinja_expressions():
    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    default_fixture_dir = os.path.join(BASE_DIR, 'expression_builder', 'expressions')
    for filename in glob(default_fixture_dir + '**/*.json'):
        default_fixture_path = os.path.join(default_fixture_dir, filename)
        try:
            with open(default_fixture_path) as json_data:
                model_data = json.load(json_data)
                insert_jinja_record(model_data, is_system=True)
        except Exception as e:
            print('Error, while inserting default data fixture for expressions: Error {0}'.format(str(e)))
            raise Exception(
                {'Error, while inserting default data fixture for expressions: Error {0}'.format(str(e))})
