""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
"""
Django settings for sealab project.

Generated by 'django-admin startproject' using Django 1.8.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import glob
import os
import sys
from configparser import RawConfigParser
from os.path import join
from utility.config_parser import all_config
from jinja2 import ChainableUndefined, Undefined
import yaql as yaq
from fsr_utilities import decrypt

ALL_CONFIG = all_config
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
FIXTURE_DIRS = [
    BASE_DIR + '/sealab/fixtures/'
]

# Load configurations
CONFIG_DIR = join(BASE_DIR, 'sealab')
CONFIG_FILE = 'config.ini'
TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
CELERY_CONFIG = {}

application_config = RawConfigParser()
try:
    application_config.read(join(CONFIG_DIR, CONFIG_FILE))
except ImportError as ie:
    raise ie

# TODO consolidate the two strategies below
if 'test' not in sys.argv:
    APP_HOST = application_config.get('application', 'APP_HOST')
    SMTP_HOST = application_config.get('email', 'SMTP_HOST')
    EMAIL_PORT = application_config.getint('email', 'EMAIL_PORT')
    EMAIL_HOST_USER = application_config.get('email', 'EMAIL_HOST_USER')
    EMAIL_HOST_PASSWORD = application_config.get('email', 'EMAIL_HOST_PASSWORD')
    EMAIL_USE_TLS = application_config.getboolean('email', 'EMAIL_USE_TLS')
    EMAIL_TIMEOUT = application_config.getint('email', 'EMAIL_TIMEOUT')
    DEFAULT_FROM_EMAIL = application_config.get('email', 'DEFAULT_FROM_EMAIL')
    DB_HOST = ALL_CONFIG.config.get('postgres').get('pg_host')
    DB_PORT = ALL_CONFIG.config.get('postgres').get('pg_port')
    DB_NAME = application_config.get('database', 'DB_NAME')
    DB_USER = ALL_CONFIG.config.get('postgres').get('pg_user')
    DB_PASSWORD = ALL_CONFIG.config.get('postgres').get('pg_password')
    CELERY_CONFIG["task_time_limit"] = application_config.getint('celeryd', 'CELERYD_TASK_TIME_LIMIT')
    CELERY_CONFIG["task_soft_time_limit"] = application_config.getint('celeryd', 'CELERYD_TASK_SOFT_TIME_LIMIT')
    COPY_ENV_FOR_REFERENCE_WORKFLOW = application_config.getboolean('celeryd', 'COPY_ENV_FOR_REFERENCE_WORKFLOW',
                                                                    fallback=True)

    CELERY_CONFIG["result_expires"] = eval(application_config.get('celeryd', 'CELERY_TASK_RESULT_EXPIRES', fallback='7200'))
    DEBUG = application_config.getboolean('application', 'DEBUG')
    ALLOWED_HOSTS = eval(application_config.get('application', 'ALLOWED_HOSTS'))
    SCHEDULES_AUTH_CHECK = application_config.getboolean('application', 'SCHEDULES_AUTH_CHECK', fallback=True)
    LOG_REQUEST_IN_WORKFLOW = application_config.getboolean('application', 'LOG_REQUEST_IN_WORKFLOW', fallback=True)
    LOG_PURGE_CHUNK_SIZE = eval(application_config.get('application', 'LOG_PURGE_CHUNK_SIZE', fallback='1000'))
    THREAD_POOL_WORKER = eval(application_config.get('application', 'THREAD_POOL_WORKER', fallback='32'))
    PARALLEL_BRANCH_THREAD_POOL = eval(
        application_config.get('application', 'PARALLEL_BRANCH_THREAD_POOL', fallback='4'))
    SYNC_DELAY_LIMIT = eval(application_config.get('application', 'SYNC_DELAY_LIMIT', fallback='60'))
    PARALLEL_PATH = application_config.getboolean('application', 'PARALLEL_PATH', fallback=True)
    KEEP_SYSTEM_NOTIFICATION_DAYS = application_config.getint('application', 'KEEP_SYSTEM_NOTIFICATION_DAYS',
                                                              fallback=14)
    KEEP_NOTIFICATION_FAILURE_LOGS_DAYS = application_config.getint('application', 'KEEP_NOTIFICATION_FAILURE_LOGS_DAYS',
                                                              fallback=14)

    CYOPS_NULL = application_config.get('application', 'CYOPS_NULL', fallback='CYOPS_NULL')
    ENABLE_CHAINABLE_UNDEFINED = application_config.getboolean('application', 'ENABLE_CHAINABLE_UNDEFINED',
                                                               fallback=True)
    BLOCK_IN_TEMPLATE = ['__class__', '__base__', '__subclass__', '__subclasses__', '__builtins__', '__import__', '__globals__', '__init__']
    ENABLE_PRIORITY = application_config.getboolean('application', 'ENABLE_PRIORITY', fallback=True)
    CELERY_CONFIG["task_ignore_result"] = application_config.getboolean('application', 'CELERY_TASK_IGNORE_RESULT', fallback=True)
    REF_SELF_PB_LOOP_LEVEL = application_config.getint('application', 'REF_SELF_PB_LOOP_LEVEL', fallback=10)
    USE_SANDBOX_ENV = application_config.getboolean('application', 'USE_SANDBOX_ENV', fallback=True)
    DISALLOWED_JINJA_FILTERS = application_config.get('application', 'DISALLOWED_JINJA_FILTERS', fallback="['readfile']")
    SELF_ID = ''
    USE_PG_SQUEEZE = application_config.getboolean('application', 'USE_PG_SQUEEZE', fallback=True)
    USE_PG_REPACK = application_config.getboolean('application', 'USE_PG_REPACK', fallback=True)
    WF_MAX_WAITING_DAYS = application_config.getint('application', 'WF_MAX_WAITING_DAYS', fallback=7)
    WF_TABLES_TO_CLEAN = application_config.get('application', 'WF_TABLES_TO_CLEAN', fallback='workflow_workflow,workflow_step').split(",")
    PG_REPACK_LOCATION = application_config.get('application', 'PG_REPACK_LOCATION', fallback='/usr/pgsql-16/bin')
    SQUEEZE_REPACK_RETRY_COUNT = application_config.getint('application', 'SQUEEZE_REPACK_RETRY_COUNT', fallback=6)
    SQUEEZE_REPACK_RETRY_WAIT = application_config.getint('application', 'SQUEEZE_REPACK_RETRY_WAIT', fallback=10)
    SQUEEZE_TASK_TIMEOUT = application_config.getint('celeryd', 'SQUEEZE_TASK_TIMEOUT', fallback=3800)
    SQUEEZE_SOFT_TASK_TIMEOUT = application_config.getint('celeryd', 'SQUEEZE_SOFT_TASK_TIMEOUT', fallback=3600)
    SQUEEZE_EXECUTION_TIME = application_config.get('application', 'SQUEEZE_EXECUTION_TIME', fallback="22-6")
    FREE_SPACE_THRESHOLD = application_config.getint('application', 'FREE_SPACE_THRESHOLD', fallback=53687091200)
    SPACE_CHECK_WAIT_TIME = application_config.getint('application', 'SPACE_CHECK_WAIT_TIME', fallback=300)
else:
    APP_HOST = 'localhost'
    SMTP_HOST = 'localhost'
    EMAIL_PORT = 25
    EMAIL_HOST_USER = ''
    EMAIL_HOST_PASSWORD = ''
    EMAIL_USE_TLS = False
    EMAIL_TIMEOUT = 30
    DEFAULT_FROM_EMAIL = 'soc@cybersponse.com'
    DB_HOST = 'localhost'
    DB_PORT = '5432'
    DB_NAME = 'sealab'
    DB_USER = 'cyberpgsql'
    DB_PASSWORD = ''
    CELERY_CONFIG["task_time_limit"] = 600
    CELERY_CONFIG["task_soft_time_limit"] = 600
    COPY_ENV_FOR_REFERENCE_WORKFLOW = False
    CELERY_CONFIG["result_expires"] = 7200
    DEBUG = False
    ALLOWED_HOSTS = []
    SCHEDULES_AUTH_CHECK = True
    LOG_REQUEST_IN_WORKFLOW = False
    PARALLEL_PATH = True
    PARALLEL_BRANCH_THREAD_POOL = 4
    CYOPS_NULL = 'CYOPS_NULL'
    ENABLE_CHAINABLE_UNDEFINED = True
    BLOCK_IN_TEMPLATE = ['__class__', '__base__', '__subclass__', '__builtins__', '__import__']
    ENABLE_PRIORITY = True
    CELERY_CONFIG["task_ignore_result"] = True
    SELF_ID = ''
    USE_SANDBOX_ENV = True
    REF_SELF_PB_LOOP_LEVEL = application_config.getint('application', 'REF_SELF_PB_LOOP_LEVEL', fallback=10)
    USE_PG_SQUEEZE = True
    USE_PG_REPACK = True
    WF_MAX_WAITING_DAYS = 7
    WF_TABLES_TO_CLEAN = ['workflow_workflow', 'workflow_step']
    PG_REPACK_LOCATION = '/usr/pgsql-16/bin'
    SQUEEZE_REPACK_RETRY_COUNT = 6
    SQUEEZE_REPACK_RETRY_WAIT = 10
    SQUEEZE_TASK_TIMEOUT = 3800
    SQUEEZE_SOFT_TASK_TIMEOUT = 3600
    SQUEEZE_EXECUTION_TIME = '22-6'
    FREE_SPACE_THRESHOLD = 53687091200
    SPACE_CHECK_WAIT_TIME = 300

# TODO: this is really stupid
# this part should go in wsgi.py and/or manage.py, but celery needs it too, and
# if it lives here then env args will always be set reguardless of how the
# application was started and doesn't have to be repeated in three different
# places.
ENV_DIR = os.path.join(BASE_DIR, '.envdir')
ENV_VARS = glob.glob(os.path.join(ENV_DIR, '*'))
for env_var in ENV_VARS:
    with open(env_var, 'r') as env_var_file:
        os.environ.setdefault(env_var.split(os.sep)[-1],
                              env_var_file.read().strip())

APPLIANCE_PUBLIC_KEY = os.getenv('APPLIANCE_PUBLIC_KEY', '')
APPLIANCE_PRIVATE_KEY = os.getenv('APPLIANCE_PRIVATE_KEY', '')
SEALAB_PUBLIC_KEY = os.getenv('SEALAB_PUBLIC_KEY', '')
SEALAB_PRIVATE_KEY = os.getenv('SEALAB_PRIVATE_KEY', '')

# RabbitMQ Publisher

RABBITMQ_CONFIG_FILE = "/opt/cyops/configs/rabbitmq/rabbitmq_users.conf"
CONNECTION_RETRY_COUNT = 3
CONNECTION_RETRY_DELAY = 5
PUBLISH_MESSAGE_RETRY_COUNT = 5
ACTION_LIMIT_ENABLED = False

# ToDo  can be moved in config file ?
QUEUE = {
    'queue_sealab_action_remoterequest': {
        'exchangeName': 'texchange.cyops.sealab',
        'routingKey': 'route.sealab.action.remoterequest'
    },
    'queue_sealab_action_remoteresponse': {
        'exchangeName': 'texchange.cyops.sealab',
        'routingKey': 'route.sealab.action.remoteresponse'
    }
}

# Auth settings
HMAC_TIME_THRESHOLD = 30
HMAC_USER = 'hmac_user'
BASIC_USER = 'basic_user'
ANONYMOUS_USER = 'anon_user'

# Celery settings
CELERY_CONFIG["result_backend"] = 'django-db'
username = ALL_CONFIG.config.get('mq').get('mq_user')
mq_password = ALL_CONFIG.config.get('mq').get('mq_password')

mq_password = decrypt(mq_password)

host = ALL_CONFIG.config.get('mq').get('mq_host')
virtual_host = ALL_CONFIG.config.get('mq').get('mq_vhost')
ssl_enabled = ALL_CONFIG.config.get('mq').get('mq_ssl_enabled', False)
if ssl_enabled:
    port = ALL_CONFIG.config.get('mq').get('mq_tls_port', 5671)
    import ssl

    CELERY_CONFIG["broker_use_ssl"] = {
        'ca_certs': ALL_CONFIG.config.get('mq').get('mq_ca_cert'),
        'cert_reqs': ssl.CERT_REQUIRED
    }
else:
    port = ALL_CONFIG.config.get('mq').get('mq_port', 5672)

CELERY_CONFIG["broker_url"] = 'amqp://' + username + ':' + mq_password + '@' + host + ':' + str(port) + '/' + virtual_host

HIGH_PRIORITY = 6
NORMAL_PRIORITY = 3
LOW_PRIORITY = 0

PRIORITY_PICKLIST_MAP = {
    'Low': LOW_PRIORITY,
    'Medium': NORMAL_PRIORITY,
    'High': HIGH_PRIORITY,

}
if ENABLE_PRIORITY:
    TASK_DEFAULT_PRIORITY = NORMAL_PRIORITY
    TASK_INHERIT_PARENT_PRIORITY = True

#: Only add pickle to this list if your broker is secured
#: from unwanted access (see userguide/security.html)
CELERY_CONFIG["accept_content"] = ['json']
CELERY_CONFIG["task_serializer"] = 'json'
CELERY_CONFIG["result_serializer"] = 'json'
CELERY_CONFIG["beat_sync_every"] = 5
CELERY_CONFIG["broker_heartbeat_checkrate"] = 1
CELERY_CONFIG["worker_max_tasks_per_child"] = 200
CELERY_CONFIG["worker_max_memory_per_child"] = 2000000  # in KB - 2GB

TEST_RUNNER = 'rainbowtests.test.runner.RainbowDiscoverRunner'

# rest framework settings
REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated'
    ],
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'rest_framework.authentication.SessionAuthentication',
        'auth_ogre.schemes.HmacAuthenticationScheme'
    ],
    'DEFAULT_RENDERER_CLASSES': [
        # 'rest_framework.renderers.BrowsableAPIRenderer',
        'rest_framework.renderers.JSONRenderer',
    ],
    # pagination
    'DEFAULT_PAGINATION_CLASS':
        'lerna.pagination.HydraLimitOffsetPagination',
    'PAGE_SIZE': 10,
    'DEFAULT_METADATA_CLASS': 'rest_framework.metadata.SimpleMetadata',
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'workflow_cache',
    }
}

ANONYMOUS_USER_ID = -1

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'yco#qath+mim6sfb&$zcfye25ph2@3725ak$ktp$$7q8^@4)@-'

# SECRET KEY TO BYPASS THE CONNECTOR RBAC
INTEGRATIONS_SECRET_KEY = 'ycoVqathYmim6sfbINzcfye25ph2@3725akAktpSS7q8^@4)@-'

# Used by DAS during restricted token creation
DAS_SECRET_KEY = '3s6#o1_iztswxq7)7scje*2!xzk8htmo4!ged^1y3++jm$ywbw'

# Application definition

INSTALLED_APPS = (
    # django
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_celery_results',

    # 3rd party
    'rest_framework',
    'django_extensions',
    'memoize',
    'django_filters',
    'crispy_forms',

    # cs
    'workflow',
    'lerna',
    'dynamic_variable',
    'ansible_filters',
    'django_celery_beat',
    'expression_builder',
    'postman',
    'rules_engine',

)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    # 'django_cprofile_middleware.middleware.ProfilerMiddleware',
)

# DJANGO_CPROFILE_MIDDLEWARE_REQUIRE_STAFF = False

ROOT_URLCONF = 'sealab.urls'

TEMPLATES = [
    {
        'NAME': 'django',
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
    {
        'NAME': 'jinja2',
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [],
        'APP_DIRS': False,
        'OPTIONS': {
            'environment': 'sealab.jinja.environment',
            'extensions': ['jinja2.ext.loopcontrols', 'jinja2.ext.do'],
            'undefined': ChainableUndefined if ENABLE_CHAINABLE_UNDEFINED else Undefined
        }
    },
]

WSGI_APPLICATION = 'sealab.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

# Decrypt DB password
if DB_PASSWORD:
    DB_PASSWORD = decrypt(DB_PASSWORD)


if 'test' in sys.argv:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'USER': 'postgres',
            'PASSWORD': DB_PASSWORD,
            'NAME': 'sealab_testz',
            'HOST': DB_HOST
        }
    }
else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'USER': DB_USER,
            'PASSWORD': DB_PASSWORD,
            'NAME': DB_NAME,
            'HOST': DB_HOST,
            'PORT': DB_PORT,
            'OPTIONS': {
                'client_encoding': 'UTF-8'
            }
        }
    }

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/
WEB_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'web/')

STATIC_URL = '/wf/static/'
STATIC_ROOT = os.path.join(WEB_ROOT, 'static/')

if DEBUG:
    DOCS_ROOT = os.path.join(BASE_DIR, '../docs/build/html/')
    STATICFILES_DIRS = (
        ('docs', DOCS_ROOT),
    )

MEDIA_URL = '/wf/media/'
MEDIA_ROOT = os.path.join(WEB_ROOT, 'media/')

TMP_FILE_ROOT = '/tmp/'

# port 9191 for fsr-workflow service
if 'uwsgi' in sys.argv or '9191' in sys.argv:
    LOGFILE_LOC = '/var/log/cyops/cyops-workflow/sealab.log'
else:
    LOGFILE_LOC = os.path.join(os.path.dirname(BASE_DIR),
                               'sealab.log')

if 'uwsgi' in sys.argv or '9191' in sys.argv:
    FSR_WORKFLOW_LOGFILE_LOC = '/var/log/cyops/cyops-workflow/fsr-workflow.log'
else:
    FSR_WORKFLOW_LOGFILE_LOC = os.path.join(os.path.dirname(BASE_DIR), 'fsr-workflow.log')

try:
    from sealab.config import WORKFLOW_LOG_LEVEL
except ImportError as ie:
    WORKFLOW_LOG_LEVEL = 'WARNING'

try:
    from sealab.config import FSR_WORKFLOW_LOG_LEVEL
except ImportError as ie:
    FSR_WORKFLOW_LOG_LEVEL = 'WARNING'

# Only configure logging when running Django, not celery
if 'worker' not in sys.argv and 'beat' not in sys.argv:
    LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'handlers': {
            # Log to a text file that can be rotated by logrotate
            'logfile': {
                'class': 'logging.handlers.WatchedFileHandler',
                'filename': LOGFILE_LOC,
                'formatter': 'verbose'
            },
            'fsr-workflow-log': {
                'class': 'logging.StreamHandler',
                'formatter': 'verbose',
            }
        },
        'formatters': {
            'verbose': {
                'format': "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
                'datefmt': TIME_FORMAT
            },
            'simple': {
                'format': '[%(asctime)s] %(levelname)s %(message)s',
                'datefmt': TIME_FORMAT
            }
        },
        'loggers': {
            'django': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'workflow': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'dynamic_variable': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'expression_builder': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'django.db.backends': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'postman': {
                'handlers': ['fsr-workflow-log'],
                'level': FSR_WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'rules_engine': {
                'handlers': ['fsr-workflow-log'],
                'level': FSR_WORKFLOW_LOG_LEVEL,
                'propagate': False
            }
        },
    }

CRUD_HUB_URL = 'https://' + APP_HOST
CRUD_HUB_API_PREFIX = '/api/3'

DEFAULT_RABBITMQ_API_PORT = '15672'

INTEGRATIONS_URL = 'https://' + APP_HOST + ':9595/integration/execute/?format=json&secretKey=' + INTEGRATIONS_SECRET_KEY

MUST_EVAL_DATA_TYPES = [dict, list, bool, type(None), str, tuple]

RMQ_PUBLISHER = None
ROUTER_HEARTBEAT = int(application_config.get('router', 'heartbeat', fallback='120'))
ROUTER_CONNECTION_ATTEMPTS = int(application_config.get('router', 'connection_attempts', fallback='5'))
ROUTER_RETRY_DELAY = int(application_config.get('router', 'retry_delay', fallback='10'))

RECOMMENDATION_PREFERENCE = None

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

YAQL_FACTORY = yaq.factory.YaqlFactory().create()
DATA_UPLOAD_MAX_MEMORY_SIZE = application_config.getint('application', 'DATA_UPLOAD_MAX_MEMORY_SIZE', fallback=2621440)
